/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Copyright (c) 2017 MediaTek Inc.
 * Author: Mars.C <mars.cheng@mediatek.com>
 */

#ifndef _DT_BINDINGS_POWER_MT6797_POWER_H
#define _DT_BINDINGS_POWER_MT6797_POWER_H

#define MT6797_POWER_DOMAIN_VDEC		0
#define MT6797_POWER_DOMAIN_VENC		1
#define MT6797_POWER_DOMAIN_ISP		2
#define MT6797_POWER_DOMAIN_MM			3
#define MT6797_POWER_DOMAIN_AUDIO		4
#define MT6797_POWER_DOMAIN_MFG_ASYNC		5
#define MT6797_POWER_DOMAIN_MFG		6
#define MT6797_POWER_DOMAIN_MFG_CORE0		7
#define MT6797_POWER_DOMAIN_MFG_CORE1		8
#define MT6797_POWER_DOMAIN_MFG_CORE2		9
#define MT6797_POWER_DOMAIN_MFG_CORE3		10
#define MT6797_POWER_DOMAIN_MJC		11

#endif /* _DT_BINDINGS_POWER_MT6797_POWER_H */
