/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
 * Copyright (c) 2013-2014 Hisilicon Limited.
 * Copyright (c) 2013-2014 Linaro Limited.
 *
 * Author: Haojian Zhuang <haojian.zhuang@linaro.org>
 */

#ifndef __DTS_HIP04_CLOCK_H
#define __DTS_HIP04_CLOCK_H

#define HIP04_NONE_CLOCK	0

/* fixed rate & fixed factor clocks */
#define HIP04_OSC50M		1
#define HIP04_CLK_50M		2
#define HIP04_CLK_168M		3

#define HIP04_NR_CLKS		64

#endif	/* __DTS_HIP04_CLOCK_H */
